@@ -1,6 +1,8 @@
 // -*- mode:C++; tab-width:8; c-basic-offset:8; indent-tabs-mode:nil -*-
-/* Blackjack - dialog.cpp
- * Copyright (C) 2003 William Jon McCann <mccann@jhu.edu>
+/* 
+ * Blackjack - dialog.cpp
+ *
+ * Copyright (C) 2003-2004 William Jon McCann <mccann@jhu.edu>
  *
  * This game is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,13 +20,12 @@
  * USA
  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <stdlib.h>
 #include <dirent.h>
-#include <gnome.h>
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
 #include <games-card-selector.h>
 #include "blackjack.h"
 #include "menu.h"
@@ -38,24 +39,32 @@
 #include <iostream>
 using namespace std;
 
-static GtkWidget *hint_dlg = NULL;
-static GtkWidget * deck_edit = NULL;
+static GtkWidget *hint_dlg  = NULL;
+static GtkWidget *deck_edit = NULL;
 
 gboolean
-get_insurance_choice ()
+get_insurance_choice (void)
 {
-        GtkWidget* dialog;
-        gchar* message;
-        gboolean choice = false;
+        GtkWidget  *dialog;
+        const char *message;
+        const char *secondary_message;
+        gboolean    choice = FALSE;
 
         message = _("Would you like insurance?");
+        secondary_message = _("Insurance is a side wager of 50%% of the original wager"
+                              " that the dealer has a natural 21 (aka blackjack) that is"
+                              " offered when the dealer's face up card is an ace. If the"
+                              " dealer has a natural 21 then the player is paid double.");
+        
+        dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel_window),
+                                         GTK_DIALOG_MODAL,
+                                         GTK_MESSAGE_QUESTION,
+                                         GTK_BUTTONS_YES_NO,
+                                         message);
+
+        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+                                                  secondary_message);
 
-        dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW (app),
-                                                     GTK_DIALOG_MODAL,
-                                                     GTK_MESSAGE_QUESTION,
-                                                     GTK_BUTTONS_YES_NO,
-                                                     "<span weight=\"bold\" size=\"larger\">%s</span>",
-                                                     message);
         gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);
 
         gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_NO);
@@ -63,12 +72,13 @@ get_insurance_choice ()
         /* add a stock icon? */ 
         switch (gtk_dialog_run (GTK_DIALOG (dialog))) {
         case GTK_RESPONSE_YES: 
-                choice = true;
+                choice = TRUE;
                 break;
         default:
-                choice = false;
+                choice = FALSE;
                 break;
         }
+
         gtk_widget_destroy (dialog);
 
         return choice;
@@ -81,26 +91,33 @@ hint_destroy_callback (void)
 }
 
 void
-show_hint_dialog ()
+show_hint_dialog (void)
 {
-        gchar *gmessage;
+        char *message           = NULL;
+        char *secondary_message = NULL;
 
-        if (bj_game_is_first_hand ())
-                gmessage = g_strdup (_("Set your wager and click in the white outline to deal a new hand."));
-        else if (! bj_game_is_active ())
-                gmessage = g_strdup (_("Set your wager or click on the cards to deal a new hand."));
-        else
-                gmessage = bj_hand_get_best_option_string ();
+        if (bj_game_is_first_hand ()) {
+                message = g_strdup (_("Deal a new hand"));
+                secondary_message = g_strdup (_("Set your wager and click in the white outline to deal a new hand."));
+        } else if (! bj_game_is_active ()) {
+                message = g_strdup (_("Deal a new hand"));
+                secondary_message = g_strdup (_("Set your wager or click on the cards to deal a new hand."));
+        } else {
+                message = bj_hand_get_best_option_string (&secondary_message);
+        }
   
         if (hint_dlg)
                 gtk_widget_destroy (GTK_WIDGET (hint_dlg));
 
-        hint_dlg = gtk_message_dialog_new_with_markup (GTK_WINDOW (app),
-                                                       GTK_DIALOG_DESTROY_WITH_PARENT,
-                                                       GTK_MESSAGE_INFO,
-                                                       GTK_BUTTONS_OK,
-                                                       "<span weight=\"bold\" size=\"larger\">%s</span>",
-                                                       gmessage);
+        hint_dlg = gtk_message_dialog_new (GTK_WINDOW (toplevel_window),
+                                           GTK_DIALOG_DESTROY_WITH_PARENT,
+                                           GTK_MESSAGE_INFO,
+                                           GTK_BUTTONS_OK,
+                                           message);
+
+        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (hint_dlg),
+                                                  secondary_message);
+
         gtk_container_set_border_width (GTK_CONTAINER (hint_dlg), 6);
 
         if (hint_dlg)
@@ -112,7 +129,8 @@ show_hint_dialog ()
 	gtk_dialog_run (GTK_DIALOG (hint_dlg));
 	gtk_widget_destroy (hint_dlg);
 
-        g_free (gmessage);
+        g_free (message);
+        g_free (secondary_message);
 }
 
 void 
@@ -169,7 +187,7 @@ select_rule_cb (GtkTreeSelection *select, gpointer data)
                 if (g_ascii_strcasecmp (filename, bj_game_get_rules_file ())) {
                         if (data != NULL)
                                 gtk_widget_set_sensitive (GTK_WIDGET (data), FALSE);
-                        bj_game_new (filename, &seed);
+                        bj_game_new (filename, NULL);
                         if (data != NULL)
                                 gtk_widget_set_sensitive (GTK_WIDGET (data), TRUE);
                 }
@@ -183,7 +201,7 @@ reset_button_cb (GtkWidget *widget, gpointer data)
 }
 
 void
-show_preferences_dialog () 
+show_preferences_dialog (void) 
 {
         static GtkWidget* pref_dialog = NULL;
         GtkWidget *frame;
@@ -203,7 +221,7 @@ show_preferences_dialog ()
 
         if (!pref_dialog) {
                 pref_dialog = gtk_dialog_new_with_buttons (_("Blackjack Preferences"),
-                                                           GTK_WINDOW (app), 
+                                                           GTK_WINDOW (toplevel_window), 
                                                            GTK_DIALOG_DESTROY_WITH_PARENT,
                                                            GTK_STOCK_CLOSE, 
                                                            GTK_RESPONSE_CLOSE,
@@ -343,8 +361,7 @@ show_preferences_dialog ()
                                                 GTK_POLICY_AUTOMATIC,
                                                 GTK_POLICY_AUTOMATIC);
                 
-                gtk_box_pack_end (GTK_BOX (vbox), scrolled_window, TRUE, TRUE,
-                                  GNOME_PAD_SMALL );
+                gtk_box_pack_end (GTK_BOX (vbox), scrolled_window, TRUE, TRUE, 6);
                 
                 g_signal_connect (select, "changed", 
                                   G_CALLBACK (select_rule_cb), (gpointer) pref_dialog);
