@@ -1,22 +1,38 @@
 /* -*- mode:C; tab-width:8; c-basic-offset:8; indent-tabs-mode:true -*- */
 
 /*
- *
+ * Mines for GNOME
  * Author:        Pista <szekeres@cyberspace.mht.bme.hu>
  *
  * Score support: horape@compendium.com.ar
  * Mine Resizing: djb@redhat.com
  *
+ * This game is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ * USA
  */
+
 #include <config.h>
 #include <gnome.h>
-#include <libgnomeui/gnome-window-icon.h>
 #include <gconf/gconf-client.h>
+#include <math.h>
 #include <string.h>
 
 #include "minefield.h"
 #include "games-clock.h"
 #include "games-frame.h"
+#include "games-scores-dialog.h"
 
 /* Limits for various minefield properties */
 #define XSIZE_MIN 4
@@ -36,6 +52,9 @@
 #define KEY_WIDTH    	       "/apps/gnomine/geometry/width"
 #define KEY_HEIGHT   	       "/apps/gnomine/geometry/height"
 
+/* So that it is only one place if we re-order the menus. */
+#define HINT_MENU_LOC 1
+
 static GtkWidget *mf_frame;
 static GtkWidget *mfield;
 static GtkWidget *pref_dialog = NULL;
@@ -72,24 +91,32 @@ char *fsize2names[] = {
 	N_("Custom"),
 };
 
+/* It's a little ugly, but it stops the hint dialogs triggering the
+ * hide-the-window-to-stop cheating thing. */
+gboolean disable_hiding = FALSE;
+
 static void create_preferences (void);
 
 
 static GtkWidget *
 image_widget_setup (char *name)
 {
-	GdkPixbuf *pixbuf = NULL;
+	/*	GdkPixbuf *pixbuf = NULL; */
+	GtkWidget *image = NULL;
 	char *filename = NULL;
 
+	image = gtk_image_new ();
 	filename = gnome_program_locate_file (NULL,
 					      GNOME_FILE_DOMAIN_APP_PIXMAP, name,
 					      TRUE, NULL);
 	if (filename != NULL)
-		pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
+		gtk_image_set_from_file (GTK_IMAGE (image), filename);
+	/*		pixbuf = gdk_pixbuf_new_from_file (filename, NULL); */
 
 	g_free (filename);
 
-	return gtk_image_new_from_pixbuf (pixbuf);
+	/*	return gtk_image_new_from_pixbuf (pixbuf); */
+	return image;
 }
 
 static void
@@ -101,6 +128,9 @@ show_face (GtkWidget *pm)
 		gtk_widget_hide (pm_current); 
 	}
 
+	gtk_widget_set_sensitive(gamemenu[HINT_MENU_LOC].widget, 
+				 (pm == pm_cool) || (pm == pm_smile));
+
 	gtk_widget_show (pm);
 	
 	pm_current = pm;
@@ -128,33 +158,33 @@ set_flabel (GtkMineField *mfield)
 	g_free (val);
 }
 
-static void
-update_score_state (void)
-{
-        gchar **names = NULL;
-        gfloat *scores = NULL;
-	time_t *scoretimes = NULL;
-	gint top;
-	gchar buf[64];
-
-	if (fsize<4)
-		strncpy (buf, fsize2names[fsize], sizeof(buf));
-
-	top = gnome_score_get_notable ("gnomine", buf, &names, &scores, &scoretimes);
-	if (top > 0) {
-		gtk_widget_set_sensitive (gamemenu[2].widget, TRUE);
-		g_strfreev (names);
-		g_free (scores);
-		g_free (scoretimes);
-	} else { 
-		gtk_widget_set_sensitive (gamemenu[2].widget, FALSE);
-	}
-}
-
 static void
 show_scores (gchar *level, guint pos)
 {
-	gnome_scores_display (_("GNOME Mines"), "gnomine", level, pos);
+	int i;
+	static GtkWidget *dialog = NULL;
+
+	if (dialog != NULL) {
+		gtk_window_present (GTK_WINDOW (dialog));
+	} else {
+		dialog = games_scores_dialog_new ("gnomine", _("GNOME Mines Scores"));
+		for (i=0; i<4; i++) {
+			games_scores_dialog_add_category (GAMES_SCORES_DIALOG (dialog),
+							  fsize2names[i],
+							  _(fsize2names[i]));
+		}
+		games_scores_dialog_set_category (GAMES_SCORES_DIALOG (dialog),
+						  fsize2names[fsize]);
+		games_scores_dialog_set_category_description (GAMES_SCORES_DIALOG (dialog),
+							      _("Size:"));	
+		games_scores_dialog_set_style (GAMES_SCORES_DIALOG (dialog),
+					       GAMES_SCORES_STYLE_TIME);
+	}
+
+	games_scores_dialog_set_hilight (GAMES_SCORES_DIALOG (dialog), pos);
+
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_hide (dialog);
 }
 
 static void
@@ -171,8 +201,6 @@ top_ten (GtkWidget *widget, gpointer data)
 static void
 new_game (GtkWidget *widget, gpointer data)
 {
-	static guint oldx = -1;
-	static guint oldy = -1;
 	gint width, height, w_diff, h_diff;
 	guint size;
 	gint x, y;
@@ -203,28 +231,58 @@ new_game (GtkWidget *widget, gpointer data)
 
 	set_flabel (GTK_MINEFIELD (mfield));
 
-	if ((oldx != -1) && (oldy != -1) &&
-	    ((oldx != x) || (oldy != y))) {
-		/* preserve the existing mine square size */
-		/* preserve any spacing around the minefield also */
-		width  = size * mf->xsize + w_diff;
-		height = size * mf->ysize + h_diff;
-		gtk_window_resize (GTK_WINDOW (window), width, height);
-	}
-	oldx = x;
-	oldy = y;
-	
 	gtk_widget_hide (ralign);
 	gtk_widget_show (mf_frame); 
 }
 
+/* Add a penalty for a successful hint. */
+static void
+hint_used (GtkWidget *widget, gpointer data)
+{
+	/* There is a ten second penalty for accepting a hint. */
+	games_clock_add_seconds (GAMES_CLOCK (clk), 10);
+}
+
+static void
+hint (GtkWidget *widget, gpointer data)
+{
+	int result;
+	gchar * message;
+	GtkWidget * dialog;
+	
+	result = gtk_minefield_hint (GTK_MINEFIELD (mfield));
+
+	/* Successful hints are handled by the callback. */
+	if (result == MINEFIELD_HINT_ACCEPTED)
+		return;
+
+	if (result == MINEFIELD_HINT_NO_GAME)
+		message = _("Click a square, any square");
+	else
+		message = _("Maybe they're all mines ...");
+
+	dialog = gtk_message_dialog_new (GTK_WINDOW (window), GTK_DIALOG_MODAL,
+					 GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
+					 message, NULL);
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+	gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label), TRUE);
+
+	disable_hiding = TRUE;
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+	disable_hiding = FALSE;
+
+}
+
 static void
 focus_out_cb (GtkWidget *widget, GdkEventFocus *event, gpointer data)
 {
-	if (GAMES_CLOCK (clk)->timer_id != -1) {
+	if ((GAMES_CLOCK (clk)->timer_id != -1) 
+	    && (!disable_hiding)) {
 		gtk_widget_hide (mf_frame);
 		gtk_widget_show (ralign);
 		gtk_widget_grab_focus (rbutton);
+		gtk_widget_set_sensitive(gamemenu[HINT_MENU_LOC].widget,FALSE);
 
 		games_clock_stop (GAMES_CLOCK (clk)); 
 	}
@@ -235,6 +293,7 @@ resume_game_cb (GtkButton *widget, gpointer data)
 {
 	gtk_widget_hide (ralign);
 	gtk_widget_show (mf_frame);
+	gtk_widget_set_sensitive(gamemenu[HINT_MENU_LOC].widget,TRUE);
 
 	games_clock_start (GAMES_CLOCK (clk));
 }
@@ -262,8 +321,6 @@ win_game (GtkWidget *widget, gpointer data)
 {
         gfloat score;
         int pos;
-        gchar buf[64];
-
 
         games_clock_stop (GAMES_CLOCK (clk));
 
@@ -275,19 +332,17 @@ win_game (GtkWidget *widget, gpointer data)
 	    score = (gfloat) (GAMES_CLOCK (clk)->stopped / 60) + 
 		    (gfloat) (GAMES_CLOCK (clk)->stopped % 60) / 100;
 
-            strncpy(buf, fsize2names[fsize], sizeof(buf));
-	    pos = gnome_score_log (score, buf, FALSE);
+	    pos = gnome_score_log (score, fsize2names[fsize], FALSE);
 	} else {
+	    /* FIXME: This is actually slightly broken by the new,
+	     * uniform high score widget which displays it as a time. */
 	    score = ((nmines * 100) / (xsize * ysize)) /
 		    (gfloat) (GAMES_CLOCK (clk)->stopped); 
 
-            strncpy(buf, fsize2names[fsize], sizeof(buf));
-	    pos = gnome_score_log (score, buf, TRUE);
+	    pos = gnome_score_log (score, fsize2names[fsize], TRUE);
 	}
 
-	update_score_state ();
-
-	show_scores (buf, pos);
+	show_scores (fsize2names[fsize], pos);
 }
 
 static void
@@ -304,24 +359,13 @@ unlook_cell (GtkWidget *widget, gpointer data)
 	show_face(pm_cool);
 }
 
-static int
-range (int val, int min, int max)
-{
-	if (val < min)
-		val = min;
-	if (val > max)
-		val = max;
-	return val;
-}
-
-
 static void
 verify_ranges (void)
 {
-	xsize    = range (xsize, XSIZE_MIN, XSIZE_MAX);
-	ysize    = range (ysize, YSIZE_MIN, YSIZE_MAX);
-	nmines   = range (nmines, 1, xsize * ysize);
-	fsize    = range (fsize,  0, 3);
+	xsize    = CLAMP (xsize, XSIZE_MIN, XSIZE_MAX);
+	ysize    = CLAMP (ysize, YSIZE_MIN, YSIZE_MAX);
+	nmines   = CLAMP (nmines, 1, xsize * ysize - 2);
+	fsize    = CLAMP (fsize,  0, 3);
 }
 
 static void
@@ -337,14 +381,21 @@ about (GtkWidget *widget, gpointer data)
 		_("Faces:"),
 		"Tuomas Kuosmanen",
 		"",
+		_("Graphics:"),
+		"Richard Hoelscher",
+		"",
 		_("Score:"),
 		"Horacio J. Pe\xc3\xb1""a",
+		"",
+		_("Resizing and SVG support:"),
+		"Steve Chaplin",
+		"Callum McKenzie",
 		NULL
 	};
 	const gchar *documenters[] = {
                 NULL
         };
-	const gchar *translator_credits = _("translator_credits");
+	const gchar *translator_credits = _("translator-credits");
 	
 	if (about) {
                 gtk_window_present (GTK_WINDOW (about));
@@ -369,12 +420,12 @@ about (GtkWidget *widget, gpointer data)
        }
 
         about = gnome_about_new (_("GNOME Mines"), VERSION,
-				 "Copyright \xc2\xa9 1997-2003 Free Software "
+				 "Copyright \xc2\xa9 1997-2004 Free Software "
 				 "Foundation, Inc.",
 				 _("A Minesweeper clone."),
 				 (const char **)authors,
 				 (const char **)documenters,
-				 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
+				 strcmp (translator_credits, "translator-credits") != 0 ? translator_credits : NULL,
 				 pixbuf);
 	
 	if (pixbuf != NULL)
@@ -400,7 +451,7 @@ gconf_key_change_cb (GConfClient *client, guint cnxn_id,
 		int i;
 		i = value ? gconf_value_get_int (value) : 16;
 		if (i != xsize) {
-			xsize = range (i, XSIZE_MIN, XSIZE_MAX);
+			xsize = CLAMP (i, XSIZE_MIN, XSIZE_MAX);
 			new_game (mfield, NULL);
 		}
 	}
@@ -408,7 +459,7 @@ gconf_key_change_cb (GConfClient *client, guint cnxn_id,
 		int i;
 		i = value ? gconf_value_get_int (value) : 16;
 		if (i != ysize) {
-			ysize = range (i, YSIZE_MIN, YSIZE_MAX);
+			ysize = CLAMP (i, YSIZE_MIN, YSIZE_MAX);
 			new_game (mfield, NULL);
 		}
 	}
@@ -416,7 +467,7 @@ gconf_key_change_cb (GConfClient *client, guint cnxn_id,
 		int i;
 		i = value ? gconf_value_get_int (value) : 40;
 		if (nmines != i) {
-			nmines = range (i, 1, xsize * ysize - 2);
+			nmines = CLAMP (i, 1, xsize * ysize - 2);
 			new_game (mfield, NULL);
 		}
 	}
@@ -424,8 +475,7 @@ gconf_key_change_cb (GConfClient *client, guint cnxn_id,
 		int i;
 		i = value ? gconf_value_get_int (value) : 0;
 		if (i != fsize) {
-			fsize = range (i, 0, 3);
-			update_score_state ();
+			fsize = CLAMP (i, 0, 3);
 			new_game (mfield, NULL);
 		}
 	}
@@ -445,7 +495,7 @@ size_radio_callback (GtkWidget *widget, gpointer data)
 	fsc = GPOINTER_TO_INT (data);
 
 	gconf_client_set_int (conf_client, KEY_MODE,
-			fsc, NULL);
+			      fsc, NULL);
 
 	gtk_widget_set_sensitive(cframe, fsc == 3);
 }
@@ -453,17 +503,17 @@ size_radio_callback (GtkWidget *widget, gpointer data)
 static void
 fix_nmines (int xsize, int ysize)
 {
-  int maxmines;
+	int maxmines;
 
-  /* Fix up the maximum number of mines so that there is always at least two
-   * free spaces. It could in theory be left at one, but that gives an
-   * instant win situation. */
-  maxmines = xsize*ysize - 2;
-  if (nmines > maxmines) {
-    gconf_client_set_int (conf_client, KEY_NMINES, maxmines, NULL);
-    gtk_spin_button_set_value (GTK_SPIN_BUTTON(mentry), maxmines);
-  }
-  gtk_spin_button_set_range (GTK_SPIN_BUTTON(mentry), 1, maxmines);
+	/* Fix up the maximum number of mines so that there is always at least two
+	 * free spaces. It could in theory be left at one, but that gives an
+	 * instant win situation. */
+	maxmines = xsize*ysize - 2;
+	if (nmines > maxmines) {
+		gconf_client_set_int (conf_client, KEY_NMINES, maxmines, NULL);
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (mentry), maxmines);
+	}
+	gtk_spin_button_set_range (GTK_SPIN_BUTTON (mentry), 1, maxmines);
 }
 
 static void
@@ -472,7 +522,7 @@ xsize_spin_cb (GtkSpinButton *spin, gpointer data)
 	int size = gtk_spin_button_get_value_as_int (spin);
 	gconf_client_set_int (conf_client, KEY_XSIZE,
 			      size, NULL);
-        fix_nmines(size,ysize);
+        fix_nmines (size,ysize);
 }
 
 static void
@@ -496,7 +546,7 @@ nmines_spin_cb (GtkSpinButton *spin, gpointer data)
 static void
 use_question_toggle_cb (GtkCheckButton *check, gpointer data)
 {
-	gboolean use_marks = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check));
+	gboolean use_marks = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check));
 	gconf_client_set_bool (conf_client, KEY_USE_QUESTION_MARKS,
 				use_marks, NULL);
 }
@@ -650,6 +700,7 @@ preferences_callback (GtkWidget *widget, gpointer data)
 
 GnomeUIInfo gamemenu[] = {
         GNOMEUIINFO_MENU_NEW_GAME_ITEM(new_game, NULL),
+	GNOMEUIINFO_MENU_HINT_ITEM(hint, NULL),
 	GNOMEUIINFO_SEPARATOR,
 	GNOMEUIINFO_MENU_SCORES_ITEM(top_ten, NULL),
 	GNOMEUIINFO_SEPARATOR,
@@ -722,16 +773,15 @@ static struct poptOption options[] = {
   {NULL, 'x', POPT_ARG_INT, &xsize, 0, N_("Width of grid"), N_("X")},
   {NULL, 'y', POPT_ARG_INT, &ysize, 0, N_("Height of grid"), N_("Y")},
   {NULL, 'n', POPT_ARG_INT, &nmines, 0, N_("Number of mines"), N_("NUMBER")},
-  {NULL, 'f', POPT_ARG_INT, &fsize, 0, NULL, NULL},
-  {NULL, 'a', POPT_ARG_INT, &xpos, 0, NULL, N_("X")},
-  {NULL, 'b', POPT_ARG_INT, &ypos, 0, NULL, N_("Y")},
+  {NULL, 'f', POPT_ARG_INT, &fsize, 0, N_("Size of the board (1=small, 3=large)"), NULL},
+  {NULL, 'a', POPT_ARG_INT, &xpos, 0, N_("X location of window"), N_("X")},
+  {NULL, 'b', POPT_ARG_INT, &ypos, 0, N_("Y location of window"), N_("Y")},
   {NULL, '\0', 0, NULL, 0}
 };
 
 int
 main (int argc, char *argv[])
 {
-        GnomeAppBar *appbar;
         GtkWidget *all_boxes;
 	GtkWidget *status_box;
 	GtkWidget *button_table;
@@ -761,13 +811,10 @@ main (int argc, char *argv[])
 	gconf_client_notify_add (conf_client, KEY_DIR,
 				 gconf_key_change_cb, NULL, NULL, NULL);
 
-	gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-gnomine.png");
+	gtk_window_set_default_icon_name ("gnome-gnomine");
 	client = gnome_master_client ();
 	g_signal_connect (G_OBJECT (client), "save_yourself",
 			  G_CALLBACK (save_state), argv[0]);
-
-	if (xpos > 0 || ypos > 0)
-	  gdk_window_move (GTK_WIDGET (window)->window, xpos, ypos);
 		
 	if (xsize == -1)
 		xsize = gconf_client_get_int (conf_client,
@@ -789,31 +836,22 @@ main (int argc, char *argv[])
 						    KEY_USE_QUESTION_MARKS,
 						    NULL);
 	width = gconf_client_get_int (conf_client, KEY_WIDTH, NULL);
-	width = width ? width : WIDTH_DEFAULT;
+	width = width > 0 ? width : WIDTH_DEFAULT;
 	height = gconf_client_get_int (conf_client, KEY_HEIGHT, NULL);
-	height = height ? height : HEIGHT_DEFAULT;
+	height = height > 0 ? height : HEIGHT_DEFAULT;
 	
 	verify_ranges ();
 
-#define ELEMENTS(x) (sizeof(x) / sizeof(x[0])) 
-	{
-		int i;
-		for (i = 0; i < ELEMENTS(mainmenu); i++)
-			mainmenu[i].label = gettext (mainmenu[i].label);
-		for (i = 0; i < ELEMENTS(gamemenu); i++)
-			gamemenu[i].label = gettext (gamemenu[i].label);
-		for (i = 0; i < ELEMENTS(helpmenu); i++)
-			helpmenu[i].label = gettext (helpmenu[i].label);
-	}
+	/* This is evil, but the normal button focus indicator 
+	 * interferes with the face (but we still want the button
+	 * to be the default). */
+	gtk_rc_parse_string ("style \"gnomine\" { GtkButton::interior-focus = 0 } class \"GtkButton\" style \"gnomine\"");
 
 	window = gnome_app_new ("gnomine", _("GNOME Mines"));
 	gnome_app_create_menus (GNOME_APP (window), mainmenu);
 	gtk_window_set_default_size (GTK_WINDOW (window), width, height);	
 	gtk_window_set_resizable (GTK_WINDOW (window), TRUE);
 
-	appbar = GNOME_APPBAR (gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_NEVER));
-	gnome_app_set_statusbar (GNOME_APP (window), GTK_WIDGET (appbar));
-	
 	g_signal_connect(G_OBJECT (window), "delete_event",
 			 G_CALLBACK (quit_game), NULL);
 	g_signal_connect(G_OBJECT (window), "focus_out_event",
@@ -823,7 +861,7 @@ main (int argc, char *argv[])
 
 	gnome_app_set_contents (GNOME_APP (window), all_boxes);
 
-	button_table = gtk_table_new (2, 3, FALSE);
+	button_table = gtk_table_new (1, 3, FALSE);
 	gtk_box_pack_start (GTK_BOX (all_boxes), button_table, TRUE, TRUE, 0);
 
 	pm_current = NULL;
@@ -831,30 +869,30 @@ main (int argc, char *argv[])
 	mbutton = gtk_button_new ();
 	g_signal_connect (G_OBJECT (mbutton), "clicked",
 			  G_CALLBACK (new_game), NULL);
-        gtk_widget_set_size_request (mbutton, 38, 38);
 	gtk_table_attach (GTK_TABLE (button_table), mbutton, 1, 2, 0, 1,
 			  0, 0, 5, 5);
 
-	face_box = gtk_vbox_new (FALSE, 0);
+	face_box = gtk_vbox_new (FALSE, 5);
 	gtk_container_add (GTK_CONTAINER (mbutton), face_box);
 	
-	pm_win     = image_widget_setup ("gnomine/face-win.xpm");
-	pm_sad     = image_widget_setup ("gnomine/face-sad.xpm");
-	pm_smile   = image_widget_setup ("gnomine/face-smile.xpm");
-	pm_cool    = image_widget_setup ("gnomine/face-cool.xpm");
-	pm_worried = image_widget_setup ("gnomine/face-worried.xpm");
+	pm_win     = image_widget_setup ("gnomine/face-win.png");
+	pm_sad     = image_widget_setup ("gnomine/face-sad.png");
+	pm_smile   = image_widget_setup ("gnomine/face-smile.png");
+	pm_cool    = image_widget_setup ("gnomine/face-cool.png");
+	pm_worried = image_widget_setup ("gnomine/face-worried.png");
 
-        gtk_box_pack_start (GTK_BOX (face_box), pm_win, FALSE, FALSE, 0);
-        gtk_box_pack_start (GTK_BOX (face_box), pm_sad, FALSE, FALSE, 0);
-        gtk_box_pack_start (GTK_BOX (face_box), pm_smile, FALSE, FALSE, 0);
-        gtk_box_pack_start (GTK_BOX (face_box), pm_cool, FALSE, FALSE, 0);
-        gtk_box_pack_start (GTK_BOX (face_box), pm_worried, FALSE, FALSE, 0);
-
-	show_face (pm_smile); 
+	gtk_box_pack_start (GTK_BOX (face_box), pm_win, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (face_box), pm_sad, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (face_box), pm_smile, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (face_box), pm_cool, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (face_box), pm_worried, FALSE, FALSE, 0);
 
 	box = gtk_vbox_new (FALSE, 0);
 	gtk_table_attach_defaults (GTK_TABLE (button_table), box, 1, 2, 1, 2);
 
+	gtk_box_pack_start (GTK_BOX (box), gtk_hseparator_new (), FALSE, FALSE,
+			    0);
+
 	mfield = gtk_minefield_new ();
 
 	/* It doesn't really matter what this widget is as long as it's a
@@ -885,14 +923,19 @@ main (int argc, char *argv[])
 			  G_CALLBACK (look_cell), NULL);
 	g_signal_connect (G_OBJECT (mfield), "unlook",
 			  G_CALLBACK (unlook_cell), NULL);
+	g_signal_connect (G_OBJECT (mfield), "hint_used",
+			  G_CALLBACK (hint_used), NULL);
+
+	gtk_box_pack_start (GTK_BOX (box), gtk_hseparator_new (), FALSE, FALSE,
+			    0);
 	
-	status_box = gtk_hbox_new (FALSE, GNOME_PAD);
-	gtk_box_pack_start (GTK_BOX (appbar), status_box, 
-			    FALSE, FALSE, 0);
+	status_box = gtk_hbox_new (TRUE, 0); 
+	gtk_box_pack_start (GTK_BOX (box), status_box, 
+			    FALSE, FALSE, GNOME_PAD);
 
 	label = gtk_label_new(_("Flags:"));
 	gtk_box_pack_start (GTK_BOX (status_box), label, 
-			    FALSE, FALSE, 0);
+			    FALSE, FALSE, 0); 
 	
 	flabel = gtk_label_new ("0");
 	gtk_box_pack_start (GTK_BOX (status_box), flabel, 
@@ -900,17 +943,20 @@ main (int argc, char *argv[])
 
 	label = gtk_label_new (_("Time:"));
 	gtk_box_pack_start (GTK_BOX (status_box), label, 
-			    FALSE, FALSE, 0);
+	FALSE, FALSE, 0);
 
         clk = games_clock_new ();
 	gtk_box_pack_start (GTK_BOX (status_box), clk, 
-			    FALSE, FALSE, 0);
-
-	update_score_state ();
+			    FALSE, FALSE, 0); 
 
 	new_game (mfield, NULL);
 
         gtk_widget_show_all (window);
+
+	/* Must be after the window has been created. */
+	if (xpos >= 0 && ypos >= 0)
+	  gdk_window_move (GTK_WIDGET (window)->window, xpos, ypos);
+
 	/* All this hiding is a bit ugly, but it's better than a
 	 * ton of gtk_widget_show calls. */
 	gtk_widget_hide (ralign);
@@ -919,6 +965,8 @@ main (int argc, char *argv[])
 	gtk_widget_hide (pm_cool);
 	gtk_widget_hide (pm_worried);
 
+	show_face (pm_smile); 
+
         gtk_main ();
 
 	return 0;
